/*
 *   kgoodstuff - a button bar a la FvwmButtons for KDE
 *   Copyright (C) 1998  Juergen Vigna
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef KGoodStuff_included
#define KGoodStuff_included

#include <qfiledlg.h>
#include <qpixmap.h>
#include <qpopmenu.h>
#include <qdrawutl.h>
#include <qtimer.h>
#include <kprocess.h>
#include <kiconloader.h>
#include <ksimpleconfig.h>
#include <X11/Xlib.h>

#include "kgoodstuffdata.h"
#include "qmybt.h"
//#include "prefmain.h"
#include "prefglob.h"
#include "prefbutton.h"

#define ID_MENU_NEW	10
#define ID_MENU_CUT	11
#define ID_MENU_COPY	12
#define ID_MENU_PASTE	13
#define ID_MENU_BOPT	14
#define ID_MENU_GOPT	15
#define ID_MENU_SAVE	16
#define ID_MENU_QUIT	17
#define ID_MENU_HELP	18
#define ID_MENU_ABOUT	19
#define ID_MENU_CLOSE	20

class KGoodStuff : public KGoodStuffData
{
    Q_OBJECT

public:
    bool
        closeChild;
    
    KGoodStuff(QWidget* parent = NULL, const char* name = NULL,
               const char *num = 0, bool child = false,
               bool closeChildFlag = false, KGoodStuff *master = NULL);
    virtual ~KGoodStuff();
    void saveOptions();
    void setIconPath(const char *newIconPath);
    bool getGeometryData(KGoodStuff *child, int *x, int *y, bool *or);
    void setButtonsGeometry();
    QWidget* parentOfSwallowed(Window w);
    void myMove(QPoint point, bool init);
    
private:
    typedef struct {
        QMyButton *button;
        bool childKGSFlag,childCloseOpt;
        KGoodStuff *childKGS;
        KDNDDropZone *dz;
        QString
            name,
            pixmap,
            cmd,
            desc,
            swallowName,
            swallowCmd;
        int
            useRows,
            useCols;
        Window swallowed;
    } MyButtonStruct;
    MyButtonStruct
        **buttons;
    QString
        iconPath,
        aConfigName,
        kconfgroup,
        kconfglob,
        maxXY, minXY, posX, posY,
        debugFlag;
    MyButtonStruct
        **emptyButton,
        *copyButton;
    QPopupMenu
        *popup_menu;
    int
        eRaise,
        lLower,
        nColumns,
        nButtons;
    bool
        isChild,
        hOrient,
        noPopupMenu;
    PreferencesGlob
        *prefglob;
    PreferencesButton
        *prefbutton;
//    PreferencesMain
//        *prefmain;
    KIconLoader
        *il;
    KSimpleConfig
        *conf;
    QFont
        *font;
    QMyButton
        *thisButton,
        *lastButton;
    KGoodStuff
        *masterKGS;
    QPoint
        mymv_offset;
    QTimer
        *timer_raise,
        *timer_lower;
    bool localWindowAdd(Window);
    void removeChilds();
    void launchSwallowApplications();
    void updateSwallowButtons();
    void changeName(const char *name);
    
public slots:
    // kwm communication
    //  void kwmInit();
    void windowAdd(Window);

protected slots:
    void handleButtons(int id);
    void handleMenu(int id);
    void ReadStderr(KProcess *, char *buffer, int buflen);
    void ReadStdout(KProcess *, char *buffer, int buflen);
    void prefGlobOK();
    void prefButtonOK();
    void slotDropEvent(KDNDDropZone * _dropZone);
    void closeKGS(KGoodStuff *kgs);
    void enterEvent(QEvent *);
    void leaveEvent(QEvent *);

signals:
    void closeButtonBar(KGoodStuff *kbb);

protected:
    void handle_popup(QPopupMenu *menu, QWidget *button);
    QPixmap loadIcon(const char *pname, bool flag=false, QMyButton *button=0);
    bool eventFilter(QObject *ob, QEvent *ev);
    void executeCommand(const char *ecmd, const char *fname=0);
    void saveGlobalOptions();
    void saveButtonOptions();
    void moveButtonPosition(QMyButton *button, int oldx, int oldy);
    void arrow_on_pixmap(QPixmap* pm, QMyButton *button);
};
#endif // KGoodStuff_included
